library example_build;

import 'dart:async';
import 'dart:io';
import 'dart:mirrors';
import 'package:ccompile/ccompile.dart';

void main() {
  var projectPath = Utils.toAbsolutePath('../example/sample_extension.yaml');
  Utils.buildProject(projectPath, {
    'start': 'Building project "$projectPath"',
    'success': 'Building complete successfully',
    'error': 'Building complete with some errors'})
    .then((exitCode) {});
}

class Utils {
  static Future<int> buildProject(projectPath, Map messages) {
    var workingDirectory = new Path(projectPath).directoryPath.toNativePath();
    return new Future.immediate(null).then((_) {
      var message = messages['start'];
      if(!message.isEmpty) {
        Utils.writeString(message, stdout);
      }

      var builder = new ProjectBuilder();
      return builder.loadProject(projectPath).then((project) {
        return builder.buildAndClean(project, workingDirectory).then((result) {
          if(result.exitCode == 0) {
            var message = messages['success'];
            if(!message.isEmpty) {
              Utils.writeString(message, stdout);
            }
          } else {
            var message = messages['error'];
            if(!message.isEmpty) {
              Utils.writeString(message, stdout);
            }

            Utils.writeString(result.stdout, stdout);
            Utils.writeString(result.stderr, stderr);
          }

          return new Future.immediate(result.exitCode == 0 ? 0 : -1);
        });
      });
    });
  }

  static String toAbsolutePath(String path) {
    return new Path(Utils.getRootScriptDirectory()).join
        (new Path(path)).toNativePath();
  }

  static String newline = Platform.operatingSystem == 'windows' ? '\r\n' : '\n';

  static void writeString(String string, IOSink stream) {
    stream.writeln('$string');
  }

  static String getRootScriptDirectory() {
    var reflection = currentMirrorSystem();
    var file = reflection.isolate.rootLibrary.url;
    if(Platform.operatingSystem == 'windows') {
      file = file.replaceAll('file:///', '');
    } else {
      file = file.replaceAll('file://', '');
    }

    return new Path(file).directoryPath.toNativePath();
  }
}
